#pragma once

// System include
#include <cstddef>

// Utils
#include "metkit/mars2grib/backend/concepts/generating-process/generatingProcessEnum.h"
#include "metkit/mars2grib/utils/generalUtils.h"

namespace metkit::mars2grib::backend::concepts_ {

template <class MarsDict_t, class OptDict_t>
std::size_t generatingProcessMatcher(const MarsDict_t& mars, const OptDict_t& opt) {
    return static_cast<std::size_t>(GeneratingProcessType::Default);
}

}  // namespace metkit::mars2grib::backend::concepts_
